@@ -21,10 +21,10 @@
 #include <libavformat/avio.h>
 #include <libavutil/audio_fifo.h>
 #include <libavutil/avstring.h>
+#include <libavutil/mem.h>
 #include <libavutil/opt.h>
 #include <libavutil/parseutils.h>
 #include <libavutil/timestamp.h>
-#include <libavutil/mem.h>
 
 #include "av_stubs.h"
 #include "avcodec_stubs.h"
@@ -69,6 +69,9 @@ typedef struct av_t {
   int header_written;
   int (*write_frame)(AVFormatContext *, AVPacket *);
   int custom_io;
+
+  // avio
+  value avio;
 } av_t;
 
 #define Av_base_val(v) (*(av_t **)Data_custom_val(v))
@@ -99,7 +102,7 @@ static void free_stream(stream_t *stream) {
   if (stream->codec_context)
     avcodec_free_context(&stream->codec_context);
 
-  free(stream);
+  av_free(stream);
 }
 
 static void close_av(av_t *av) {
@@ -115,7 +118,7 @@ static void close_av(av_t *av) {
         if (av->streams[i])
           free_stream(av->streams[i]);
       }
-      free(av->streams);
+      av_free(av->streams);
       av->streams = NULL;
     }
 
@@ -138,19 +141,19 @@ static void close_av(av_t *av) {
 
   caml_acquire_runtime_system();
 
-  if (av->control_message_callback) {
+  if (av->control_message_callback)
     caml_remove_generational_global_root(&av->control_message_callback);
-  }
 
-  if (av->interrupt_cb != Val_none) {
+  if (av->interrupt_cb)
     caml_remove_generational_global_root(&av->interrupt_cb);
-    av->interrupt_cb = Val_none;
-  }
+
+  if (av->avio)
+    caml_remove_generational_global_root(&av->avio);
 
   av->closed = 1;
 }
 
-static void finalize_av(value v) { free(Av_base_val(v)); }
+static void finalize_av(value v) { av_free(Av_base_val(v)); }
 
 static struct custom_operations av_ops = {
     "ocaml_av_context",       finalize_av,
@@ -164,7 +167,7 @@ AVFormatContext *ocaml_av_get_format_context(value *p_av) {
 CAMLprim value ocaml_av_container_options(value unit) {
   CAMLparam0();
   CAMLlocal1(ret);
-  CAMLreturn(value_of_avclass(ret, avformat_get_class()));
+  CAMLreturn(value_of_avclass(&ret, avformat_get_class()));
 }
 
 CAMLprim value ocaml_av_get_streams(value _av, value _media_type) {
@@ -314,82 +317,49 @@ void ocaml_av_set_control_message_callback(value *p_av,
 
 /***** AVIO *****/
 
+#define BUFLEN 1024
+#define MIN(a, b) ((a) < (b) ? (a) : (b))
+
 typedef struct avio_t {
-  AVFormatContext *format_context;
   AVIOContext *avio_context;
+  value buffer;
   value read_cb;
   value write_cb;
   value seek_cb;
 } avio_t;
 
-#define Avio_val(v) (*(avio_t **)Data_abstract_val(v))
-
 static int ocaml_avio_read_callback(void *private, uint8_t *buf, int buf_size) {
-  value buffer, res;
+  value res;
   avio_t *avio = (avio_t *)private;
-  int ret;
-  size_t exn_len;
+  int len = MIN(BUFLEN, buf_size);
   char *caml_exn = NULL;
-  char *c_exn = NULL;
-
-  ret = caml_c_thread_register();
 
   caml_acquire_runtime_system();
 
-  buffer = caml_alloc_string(buf_size);
-
-  caml_register_generational_global_root(&buffer);
-
   res =
-      caml_callback3_exn(avio->read_cb, buffer, Val_int(0), Val_int(buf_size));
+      caml_callback3_exn(avio->read_cb, avio->buffer, Val_int(0), Val_int(len));
   if (Is_exception_result(res)) {
     res = Extract_exception(res);
 
     caml_exn = caml_format_exception(res);
-    if (caml_exn) {
-      exn_len = strlen(caml_exn) + 1;
-      c_exn = malloc(exn_len);
-      if (!c_exn)
-        caml_raise_out_of_memory();
-      memcpy(c_exn, caml_exn, exn_len);
-      caml_stat_free(caml_exn);
-    }
+    av_log(avio->avio_context, AV_LOG_ERROR,
+           "Error while executing OCaml read callback: %s\n", caml_exn);
+    caml_stat_free(caml_exn);
 
-    if (c_exn) {
-      av_log(avio->avio_context, AV_LOG_ERROR,
-             "Error while executing OCaml read callback: %s\n", c_exn);
-      free(c_exn);
-    }
-
-    caml_remove_generational_global_root(&buffer);
     caml_release_runtime_system();
 
-    if (ret != 0) {
-      caml_c_thread_unregister();
-    }
-
     return AVERROR_EXTERNAL;
   }
 
   if (Int_val(res) < 0) {
-    caml_remove_generational_global_root(&buffer);
     caml_release_runtime_system();
-    if (ret != 0) {
-      caml_c_thread_unregister();
-    }
     return Int_val(res);
   }
 
-  memcpy(buf, String_val(buffer), Int_val(res));
-
-  caml_remove_generational_global_root(&buffer);
+  memcpy(buf, String_val(avio->buffer), Int_val(res));
 
   caml_release_runtime_system();
 
-  if (ret != 0) {
-    caml_c_thread_unregister();
-  }
-
   if (Int_val(res) == 0) {
     return AVERROR_EOF;
   }
@@ -404,62 +374,32 @@ static int ocaml_avio_write_callback(void *private, uint8_t *buf,
 static int ocaml_avio_write_callback(void *private, const uint8_t *buf,
                                      int buf_size) {
 #endif
-  value buffer, res;
+  value res;
   avio_t *avio = (avio_t *)private;
-  int ret;
-  size_t exn_len;
+  int len = MIN(BUFLEN, buf_size);
   char *caml_exn = NULL;
-  char *c_exn = NULL;
-
-  ret = caml_c_thread_register();
 
   caml_acquire_runtime_system();
 
-  buffer = caml_alloc_string(buf_size);
+  memcpy(Bytes_val(avio->buffer), buf, len);
 
-  caml_register_generational_global_root(&buffer);
-
-  memcpy(Bytes_val(buffer), buf, buf_size);
-
-  res =
-      caml_callback3_exn(avio->write_cb, buffer, Val_int(0), Val_int(buf_size));
+  res = caml_callback3_exn(avio->write_cb, avio->buffer, Val_int(0),
+                           Val_int(len));
   if (Is_exception_result(res)) {
     res = Extract_exception(res);
 
     caml_exn = caml_format_exception(res);
-    if (caml_exn) {
-      exn_len = strlen(caml_exn) + 1;
-      c_exn = malloc(exn_len);
-      if (!c_exn)
-        caml_raise_out_of_memory();
-      memcpy(c_exn, caml_exn, exn_len);
-      caml_stat_free(caml_exn);
-    }
+    av_log(avio->avio_context, AV_LOG_ERROR,
+           "Error while executing OCaml write callback: %s\n", caml_exn);
+    caml_stat_free(caml_exn);
 
-    if (c_exn) {
-      av_log(avio->avio_context, AV_LOG_ERROR,
-             "Error while executing OCaml write callback: %s\n", c_exn);
-      free(c_exn);
-    }
-
-    caml_remove_generational_global_root(&buffer);
     caml_release_runtime_system();
 
-    if (ret != 0) {
-      caml_c_thread_unregister();
-    }
-
     return AVERROR_EXTERNAL;
   }
 
-  caml_remove_generational_global_root(&buffer);
-
   caml_release_runtime_system();
 
-  if (ret != 0) {
-    caml_c_thread_unregister();
-  }
-
   return Int_val(res);
 }
 
@@ -467,7 +407,7 @@ static int64_t ocaml_avio_seek_callback(void *private, int64_t offset,
                                         int whence) {
   value res;
   avio_t *avio = (avio_t *)private;
-  int _whence, ret;
+  int _whence;
   int64_t n;
 
   switch (whence) {
@@ -484,8 +424,6 @@ static int64_t ocaml_avio_seek_callback(void *private, int64_t offset,
     return -1;
   }
 
-  ret = caml_c_thread_register();
-
   caml_acquire_runtime_system();
 
   res = caml_callback2(avio->seek_cb, Val_int(offset), Val_int(_whence));
@@ -494,34 +432,42 @@ static int64_t ocaml_avio_seek_callback(void *private, int64_t offset,
 
   caml_release_runtime_system();
 
-  if (ret != 0) {
-    caml_c_thread_unregister();
-  }
-
   return n;
 }
 
 static int ocaml_av_interrupt_callback(void *private) {
   value res;
   av_t *av = (av_t *)private;
-  int ret, n;
+  int n;
 
-  ret = caml_c_thread_register();
+  if (!av->interrupt_cb)
+    return 0;
 
   caml_acquire_runtime_system();
   res = caml_callback(av->interrupt_cb, Val_unit);
   n = Int_val(res);
   caml_release_runtime_system();
 
-  if (ret != 0) {
-    caml_c_thread_unregister();
-  }
-
   return n;
 };
 
-CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
-                                  value _write_cb, value _seek_cb) {
+#define Avio_val(v) (*(avio_t **)Data_custom_val(v))
+
+static void finalize_avio(value v) {
+  avio_t *avio = Avio_val(v);
+
+  av_free(avio->avio_context->buffer);
+  avio_context_free(&avio->avio_context);
+  av_free(avio);
+}
+
+static struct custom_operations avio_ops = {
+    "ocaml_avio_context",     finalize_avio,
+    custom_compare_default,   custom_hash_default,
+    custom_serialize_default, custom_deserialize_default};
+
+CAMLprim value ocaml_av_create_io(value _read_cb, value _write_cb,
+                                  value _seek_cb) {
   CAMLparam3(_read_cb, _write_cb, _seek_cb);
   CAMLlocal1(ret);
 
@@ -536,27 +482,21 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
   unsigned char *buffer;
   int buffer_size;
 
-  avio_t *avio = (avio_t *)calloc(1, sizeof(avio_t));
+  avio_t *avio = (avio_t *)av_mallocz(sizeof(avio_t));
   if (!avio)
     caml_raise_out_of_memory();
 
+  avio->buffer = caml_alloc_string(BUFLEN);
+  caml_register_generational_global_root(&avio->buffer);
+
   avio->read_cb = (value)NULL;
   avio->write_cb = (value)NULL;
   avio->seek_cb = (value)NULL;
 
-  avio->format_context = avformat_alloc_context();
-
-  if (!avio->format_context) {
-    free(avio);
-    caml_raise_out_of_memory();
-  }
-
-  buffer_size = Int_val(bufsize);
-  buffer = av_malloc(buffer_size);
+  buffer = av_malloc(BUFLEN);
 
   if (!buffer) {
-    avformat_free_context(avio->format_context);
-    free(avio);
+    av_free(avio);
     caml_raise_out_of_memory();
   }
 
@@ -579,11 +519,12 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
     seek_cb = ocaml_avio_seek_callback;
   }
 
-  avio->avio_context =
-      avio_alloc_context(buffer, buffer_size, write_flag, (void *)avio, read_cb,
-                         write_cb, seek_cb);
+  avio->avio_context = avio_alloc_context(
+      buffer, BUFLEN, write_flag, (void *)avio, read_cb, write_cb, seek_cb);
 
   if (!avio->avio_context) {
+    caml_remove_generational_global_root(&avio->buffer);
+
     if (avio->read_cb)
       caml_remove_generational_global_root(&avio->read_cb);
 
@@ -594,27 +535,21 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
       caml_remove_generational_global_root(&avio->seek_cb);
 
     av_freep(buffer);
-    avformat_free_context(avio->format_context);
-    free(avio);
+    av_free(avio);
     caml_raise_out_of_memory();
   }
 
-  avio->format_context->pb = avio->avio_context;
-
-  ret = caml_alloc(1, Abstract_tag);
-
+  ret = caml_alloc_custom(&avio_ops, sizeof(avio_t *), 0, 1);
   Avio_val(ret) = avio;
 
   CAMLreturn(ret);
 }
 
-CAMLprim value caml_av_input_io_finalise(value _avio) {
+CAMLprim value caml_av_io_close(value _avio) {
   CAMLparam1(_avio);
   avio_t *avio = Avio_val(_avio);
 
-  // format_context is freed as part of close_av.
-  av_free(avio->avio_context->buffer);
-  avio_context_free(&avio->avio_context);
+  caml_remove_generational_global_root(&avio->buffer);
 
   if (avio->read_cb)
     caml_remove_generational_global_root(&avio->read_cb);
@@ -625,15 +560,13 @@ CAMLprim value caml_av_input_io_finalise(value _avio) {
   if (avio->seek_cb)
     caml_remove_generational_global_root(&avio->seek_cb);
 
-  free(avio);
-
   CAMLreturn(Val_unit);
 }
 
 /***** AVInputFormat *****/
 
-void value_of_inputFormat(avioformat_const AVInputFormat *inputFormat,
-                          value *p_value) {
+void value_of_inputFormat(value *p_value,
+                          avioformat_const AVInputFormat *inputFormat) {
   if (!inputFormat)
     Fail("Empty input format");
 
@@ -654,12 +587,12 @@ CAMLprim value ocaml_av_find_input_format(value _short_name) {
   avioformat_const AVInputFormat *format = av_find_input_format(short_name);
   caml_acquire_runtime_system();
 
-  free(short_name);
+  av_free(short_name);
 
   if (!format)
     caml_raise_not_found();
 
-  value_of_inputFormat(format, &ret);
+  value_of_inputFormat(&ret, format);
 
   CAMLreturn(ret);
 }
@@ -681,10 +614,10 @@ static av_t *open_input(char *url, avioformat_const AVInputFormat *format,
                         AVDictionary **options) {
   int err;
 
-  av_t *av = (av_t *)calloc(1, sizeof(av_t));
+  av_t *av = (av_t *)av_mallocz(sizeof(av_t));
   if (!av) {
     if (url)
-      free(url);
+      av_free(url);
     caml_raise_out_of_memory();
   }
 
@@ -696,8 +629,8 @@ static av_t *open_input(char *url, avioformat_const AVInputFormat *format,
 
   if (!av->format_context) {
     if (url)
-      free(url);
-    free(av);
+      av_free(url);
+    av_free(av);
     caml_raise_out_of_memory();
   }
 
@@ -712,8 +645,6 @@ static av_t *open_input(char *url, avioformat_const AVInputFormat *format,
     av->format_context->interrupt_callback.callback =
         ocaml_av_interrupt_callback;
     av->format_context->interrupt_callback.opaque = (void *)av;
-  } else {
-    av->interrupt_cb = Val_none;
   }
 
   caml_release_runtime_system();
@@ -721,13 +652,11 @@ static av_t *open_input(char *url, avioformat_const AVInputFormat *format,
   caml_acquire_runtime_system();
 
   if (err < 0) {
-    if (av->interrupt_cb != Val_none) {
+    if (av->interrupt_cb)
       caml_remove_generational_global_root(&av->interrupt_cb);
-      av->interrupt_cb = Val_none;
-    }
-    free(av);
+    av_free(av);
     if (url)
-      free(url);
+      av_free(url);
     av_dict_free(options);
     ocaml_avutil_raise_error(err);
   }
@@ -739,13 +668,11 @@ static av_t *open_input(char *url, avioformat_const AVInputFormat *format,
 
   if (err < 0) {
     avformat_close_input(&av->format_context);
-    if (av->interrupt_cb != Val_none) {
+    if (av->interrupt_cb)
       caml_remove_generational_global_root(&av->interrupt_cb);
-      av->interrupt_cb = Val_none;
-    }
-    free(av);
+    av_free(av);
     if (url)
-      free(url);
+      av_free(url);
     av_dict_free(options);
     ocaml_avutil_raise_error(err);
   }
@@ -791,7 +718,7 @@ CAMLprim value ocaml_av_open_input(value _url, value _format, value _interrupt,
   av_t *av = open_input(url, format, NULL, _interrupt, &options);
 
   if (url)
-    free(url);
+    av_free(url);
 
   // Return unused keys
   count = av_dict_count(options);
@@ -823,6 +750,7 @@ CAMLprim value ocaml_av_open_input_stream(value _avio, value _format,
   avio_t *avio = Avio_val(_avio);
   avioformat_const AVInputFormat *format = NULL;
   AVDictionary *options = NULL;
+  AVFormatContext *format_context;
   char *key, *val;
   int len = Wosize_val(_opts);
   int i, err, count;
@@ -841,8 +769,18 @@ CAMLprim value ocaml_av_open_input_stream(value _avio, value _format,
   if (_format != Val_none)
     format = InputFormat_val(Some_val(_format));
 
+  format_context = avformat_alloc_context();
+
+  if (!format_context)
+    caml_raise_out_of_memory();
+
+  format_context->pb = avio->avio_context;
+
   // open input format
-  av_t *av = open_input(NULL, format, avio->format_context, Val_none, &options);
+  av_t *av = open_input(NULL, format, format_context, Val_none, &options);
+
+  av->avio = _avio;
+  caml_register_generational_global_root(&av->avio);
 
   // Return unused keys
   count = av_dict_count(options);
@@ -870,7 +808,7 @@ CAMLprim value ocaml_av_open_input_stream(value _avio, value _format,
 CAMLprim value ocaml_av_input_obj(value _av) {
   CAMLparam1(_av);
   CAMLlocal1(ret);
-  CAMLreturn(value_of_avobj(ret, Av_val(_av)->format_context));
+  CAMLreturn(value_of_avobj(&ret, Av_val(_av)->format_context));
 }
 
 CAMLprim value ocaml_av_get_metadata(value _av, value _stream_index) {
@@ -930,8 +868,8 @@ static stream_t **allocate_input_context(av_t *av) {
     Fail("Failed to read closed input");
 
   // Allocate streams context array
-  av->streams =
-      (stream_t **)calloc(av->format_context->nb_streams, sizeof(stream_t *));
+  av->streams = (stream_t **)av_calloc(av->format_context->nb_streams,
+                                       sizeof(stream_t *));
   if (!av->streams)
     caml_raise_out_of_memory();
 
@@ -949,7 +887,7 @@ static stream_t *allocate_stream_context(av_t *av, int index,
            av_get_media_type_string(type));
   }
 
-  stream_t *stream = (stream_t *)calloc(1, sizeof(stream_t));
+  stream_t *stream = (stream_t *)av_mallocz(sizeof(stream_t));
   if (!stream)
     caml_raise_out_of_memory();
 
@@ -999,7 +937,7 @@ static stream_t *open_stream_index(av_t *av, int index, const AVCodec *dec) {
   err = avcodec_parameters_to_context(stream->codec_context, dec_param);
 
   if (err < 0) {
-    free(stream);
+    av_free(stream);
     ocaml_avutil_raise_error(err);
   }
 
@@ -1009,7 +947,7 @@ static stream_t *open_stream_index(av_t *av, int index, const AVCodec *dec) {
   caml_acquire_runtime_system();
 
   if (err < 0) {
-    free(stream);
+    av_free(stream);
     ocaml_avutil_raise_error(err);
   }
 
@@ -1041,8 +979,8 @@ static int decode_packet(av_t *av, stream_t *stream, AVPacket *packet,
   if (dec->codec_type == AVMEDIA_TYPE_AUDIO ||
       dec->codec_type == AVMEDIA_TYPE_VIDEO) {
 
-    // Assumption: each time this function is called with `frames_pending == 0`,
-    // a fresh packet is also provided and no packet otherwise.
+    // Assumption: each time this function is called with `frames_pending ==
+    // 0`, a fresh packet is also provided and no packet otherwise.
     if (!av->frames_pending) {
       ret = avcodec_send_packet(dec, packet);
 
@@ -1213,7 +1151,7 @@ CAMLprim value ocaml_av_read_input(value _packet, value _frame, value _av) {
     // Assign OCaml values right away to account for potential exceptions
     // raised below.
     if (stream->codec_context->codec_type == AVMEDIA_TYPE_SUBTITLE) {
-      frame = (AVFrame *)calloc(1, sizeof(AVSubtitle));
+      frame = (AVFrame *)av_mallocz(sizeof(AVSubtitle));
 
       if (!frame) {
         av_packet_free(&packet);
@@ -1221,7 +1159,7 @@ CAMLprim value ocaml_av_read_input(value _packet, value _frame, value _av) {
       }
 
       frame_kind = PVV_Subtitle_frame;
-      frame_value = value_of_subtitle((AVSubtitle *)frame);
+      value_of_subtitle(&frame_value, (AVSubtitle *)frame);
     } else {
       frame = av_frame_alloc();
 
@@ -1235,7 +1173,7 @@ CAMLprim value ocaml_av_read_input(value _packet, value _frame, value _av) {
       else
         frame_kind = PVV_Video_frame;
 
-      frame_value = value_of_frame(frame);
+      value_of_frame(&frame_value, frame);
     }
 
     ret = decode_packet(av, stream, packet, frame);
@@ -1330,21 +1268,19 @@ CAMLprim value ocaml_av_seek_bytecode(value *argv, int argn) {
 
 /***** AVOutputFormat *****/
 
-value value_of_outputFormat(avioformat_const AVOutputFormat *outputFormat) {
-  value v;
+void value_of_outputFormat(value *ret,
+                           avioformat_const AVOutputFormat *outputFormat) {
   if (!outputFormat)
     Fail("Empty output format");
 
-  v = caml_alloc(1, Abstract_tag);
-  OutputFormat_val(v) = outputFormat;
-
-  return v;
+  *ret = caml_alloc(1, Abstract_tag);
+  OutputFormat_val(*ret) = outputFormat;
 }
 
 CAMLprim value ocaml_av_output_format_guess(value _short_name, value _filename,
                                             value _mime) {
   CAMLparam3(_short_name, _filename, _mime);
-  CAMLlocal1(ans);
+  CAMLlocal2(ans, tmp);
   char *short_name = NULL;
   char *filename = NULL;
   char *mime = NULL;
@@ -1361,7 +1297,7 @@ CAMLprim value ocaml_av_output_format_guess(value _short_name, value _filename,
     filename = av_strndup(String_val(_filename), caml_string_length(_filename));
     if (!filename) {
       if (short_name)
-        free(short_name);
+        av_free(short_name);
       caml_raise_out_of_memory();
     }
   }
@@ -1370,9 +1306,9 @@ CAMLprim value ocaml_av_output_format_guess(value _short_name, value _filename,
     mime = av_strndup(String_val(_mime), caml_string_length(_mime));
     if (!mime) {
       if (short_name)
-        free(short_name);
+        av_free(short_name);
       if (filename)
-        free(filename);
+        av_free(filename);
       caml_raise_out_of_memory();
     }
   }
@@ -1382,17 +1318,18 @@ CAMLprim value ocaml_av_output_format_guess(value _short_name, value _filename,
   caml_acquire_runtime_system();
 
   if (short_name)
-    free(short_name);
+    av_free(short_name);
   if (filename)
-    free(filename);
+    av_free(filename);
   if (mime)
-    free(mime);
+    av_free(mime);
 
   if (!guessed)
     CAMLreturn(Val_none);
 
   ans = caml_alloc_tuple(1);
-  Store_field(ans, 0, value_of_outputFormat(guessed));
+  value_of_outputFormat(&tmp, guessed);
+  Store_field(ans, 0, tmp);
 
   CAMLreturn(ans);
 }
@@ -1433,11 +1370,11 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
   int ret;
   AVIOInterruptCB interrupt_cb = {ocaml_av_interrupt_callback, NULL};
   AVIOInterruptCB *interrupt_cb_ptr = NULL;
-  av_t *av = (av_t *)calloc(1, sizeof(av_t));
+  av_t *av = (av_t *)av_mallocz(sizeof(av_t));
 
   if (!av) {
     if (file_name)
-      free(file_name);
+      av_free(file_name);
     av_dict_free(options);
     caml_raise_out_of_memory();
   }
@@ -1455,8 +1392,6 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
     caml_register_generational_global_root(&av->interrupt_cb);
     interrupt_cb.opaque = (void *)av;
     interrupt_cb_ptr = &interrupt_cb;
-  } else {
-    av->interrupt_cb = Val_none;
   }
 
   ret = avformat_alloc_output_context2(&av->format_context, format, NULL,
@@ -1464,9 +1399,9 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
 
   if (ret < 0) {
     if (file_name)
-      free(file_name);
+      av_free(file_name);
     av_dict_free(options);
-    free(av);
+    av_free(av);
 
     ocaml_avutil_raise_error(ret);
   }
@@ -1474,17 +1409,15 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
   ret = av_opt_set_dict(av->format_context, options);
 
   if (ret < 0) {
-    if (av->interrupt_cb != Val_none) {
+    if (av->interrupt_cb)
       caml_remove_generational_global_root(&av->interrupt_cb);
-      av->interrupt_cb = Val_none;
-    }
 
-    free(av);
+    av_free(av);
     if (file_name)
-      free(file_name);
+      av_free(file_name);
 
     av_dict_free(options);
-    free(av);
+    av_free(av);
 
     ocaml_avutil_raise_error(ret);
   }
@@ -1493,9 +1426,9 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
     ret = av_opt_set_dict(av->format_context->priv_data, options);
 
     if (ret < 0) {
-      free(av);
+      av_free(av);
       if (file_name)
-        free(file_name);
+        av_free(file_name);
 
       av_dict_free(options);
       ocaml_avutil_raise_error(ret);
@@ -1505,9 +1438,9 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
   // open the output file, if needed
   if (avio_context) {
     if (av->format_context->oformat->flags & AVFMT_NOFILE) {
-      free(av);
+      av_free(av);
       if (file_name)
-        free(file_name);
+        av_free(file_name);
       av_dict_free(options);
 
       av_dict_free(options);
@@ -1524,14 +1457,12 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
       caml_acquire_runtime_system();
 
       if (err < 0) {
-        if (av->interrupt_cb != Val_none) {
+        if (av->interrupt_cb)
           caml_remove_generational_global_root(&av->interrupt_cb);
-          av->interrupt_cb = Val_none;
-        }
 
-        free(av);
+        av_free(av);
         if (file_name)
-          free(file_name);
+          av_free(file_name);
         av_dict_free(options);
 
         av_dict_free(options);
@@ -1543,7 +1474,7 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
   }
 
   if (file_name)
-    free(file_name);
+    av_free(file_name);
 
   return av;
 }
@@ -1677,6 +1608,9 @@ CAMLprim value ocaml_av_open_output_stream(value _format, value _avio,
   av_t *av = open_output(format, NULL, avio->avio_context, Val_none,
                          Bool_val(_interleaved), &options);
 
+  av->avio = _avio;
+  caml_register_generational_global_root(&av->avio);
+
   // Return unused keys
   count = av_dict_count(options);
 
@@ -1751,7 +1685,7 @@ static stream_t *new_stream(av_t *av, const AVCodec *codec) {
   // Allocate streams array
   size_t streams_size =
       sizeof(stream_t *) * (av->format_context->nb_streams + 1);
-  stream_t **streams = (stream_t **)realloc(av->streams, streams_size);
+  stream_t **streams = (stream_t **)av_realloc(av->streams, streams_size);
   if (!streams)
     caml_raise_out_of_memory();
 
@@ -2201,12 +2135,7 @@ static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
 
     if (packet->flags & AV_PKT_FLAG_KEY && _on_keyframe != Val_none) {
       caml_acquire_runtime_system();
-
-      if (ret < 0)
-        ocaml_avutil_raise_error(ret);
-
       caml_callback(Field(_on_keyframe, 0), Val_unit);
-
       caml_release_runtime_system();
     }
 
@@ -2415,7 +2344,7 @@ uint8_t *ocaml_av_ff_nal_unit_extract_rbsp(const uint8_t *src, uint32_t src_len,
   uint8_t *dst;
   uint32_t i, len;
 
-  dst = av_malloc(src_len + AV_INPUT_BUFFER_PADDING_SIZE);
+  dst = av_mallocz(src_len + AV_INPUT_BUFFER_PADDING_SIZE);
   if (!dst)
     return NULL;
 
