@@ -31,39 +31,16 @@ value ocaml_avcodec_init(value unit) {
   return Val_unit;
 }
 
-static value value_of_audio_codec_id(enum AVCodecID id) {
-  value ret = Val_AudioCodecID(id);
-  return ret;
-}
-
-static value value_of_video_codec_id(enum AVCodecID id) {
-  value ret = Val_VideoCodecID(id);
-  return ret;
-}
-
-static value value_of_subtitle_codec_id(enum AVCodecID id) {
-  value ret = Val_SubtitleCodecID(id);
-  return ret;
-}
-
-static value value_of_unknown_codec_id(enum AVCodecID id) {
-  value ret = Val_UnknownCodecID(id);
-  return ret;
-}
-
 CAMLprim value ocaml_avcodec_flag_qscale(value unit) {
-  CAMLparam0();
-  CAMLreturn(Val_int(AV_CODEC_FLAG_QSCALE));
+  return Val_int(AV_CODEC_FLAG_QSCALE);
 }
 
-CAMLprim value ocaml_avcodec_get_input_buffer_padding_size() {
-  CAMLparam0();
-  CAMLreturn(Val_int(AV_INPUT_BUFFER_PADDING_SIZE));
+CAMLprim value ocaml_avcodec_get_input_buffer_padding_size(value unit) {
+  return Val_int(AV_INPUT_BUFFER_PADDING_SIZE);
 }
 
 CAMLprim value ocaml_avcodec_subtitle_codec_id_to_AVCodecID(value _codec_id) {
-  CAMLparam1(_codec_id);
-  CAMLreturn(Val_int(SubtitleCodecID_val(_codec_id)));
+  return Val_int(SubtitleCodecID_val(_codec_id));
 }
 
 /***** AVCodecContext *****/
@@ -482,7 +459,7 @@ static void free_parser(parser_t *parser) {
   if (parser->codec_context)
     avcodec_free_context(&parser->codec_context);
 
-  free(parser);
+  av_free(parser);
 }
 
 static void finalize_parser(value v) { free_parser(Parser_val(v)); }
@@ -494,7 +471,7 @@ static struct custom_operations parser_ops = {
 
 static parser_t *create_parser(AVCodecParameters *params,
                                const AVCodec *codec) {
-  parser_t *parser = (parser_t *)calloc(1, sizeof(parser_t));
+  parser_t *parser = (parser_t *)av_mallocz(sizeof(parser_t));
   if (!parser)
     caml_raise_out_of_memory();
 
@@ -591,7 +568,7 @@ static void finalize_codec_context(value v) {
   if (ctx->codec_context)
     avcodec_free_context(&ctx->codec_context);
 
-  free(ctx);
+  av_free(ctx);
 }
 
 static struct custom_operations codec_context_ops = {
@@ -608,7 +585,7 @@ CAMLprim value ocaml_avcodec_create_decoder(value _params, value _codec) {
   if (_params != Val_none)
     params = CodecParameters_val(Field(_params, 0));
 
-  codec_context_t *ctx = (codec_context_t *)calloc(1, sizeof(codec_context_t));
+  codec_context_t *ctx = (codec_context_t *)av_mallocz(sizeof(codec_context_t));
   if (!ctx)
     caml_raise_out_of_memory();
 
@@ -685,7 +662,7 @@ CAMLprim value ocaml_avcodec_create_audio_encoder(value _sample_fmt,
     }
   }
 
-  codec_context_t *ctx = (codec_context_t *)calloc(1, sizeof(codec_context_t));
+  codec_context_t *ctx = (codec_context_t *)av_mallocz(sizeof(codec_context_t));
   if (!ctx)
     caml_raise_out_of_memory();
 
@@ -767,7 +744,7 @@ CAMLprim value ocaml_avcodec_create_video_encoder(value _device_context,
     }
   }
 
-  codec_context_t *ctx = (codec_context_t *)calloc(1, sizeof(codec_context_t));
+  codec_context_t *ctx = (codec_context_t *)av_mallocz(sizeof(codec_context_t));
   if (!ctx) {
     av_dict_free(&options);
     caml_raise_out_of_memory();
@@ -910,7 +887,7 @@ CAMLprim value ocaml_avcodec_receive_frame(value _ctx) {
     ans = Val_int(0);
   } else {
     ans = caml_alloc(1, 0);
-    val_frame = value_of_frame(frame);
+    value_of_frame(&val_frame, frame);
     Store_field(ans, 0, val_frame);
   }
 
@@ -1066,19 +1043,19 @@ CAMLprim value ocaml_avcodec_parameters_get_bit_rate(value _cp) {
 CAMLprim value ocaml_avcodec_get_audio_codec_id(value _codec) {
   CAMLparam1(_codec);
   const AVCodec *codec = AvCodec_val(_codec);
-  CAMLreturn(value_of_audio_codec_id(codec->id));
+  CAMLreturn(Val_AudioCodecID(codec->id));
 }
 
 CAMLprim value ocaml_avcodec_get_video_codec_id(value _codec) {
   CAMLparam1(_codec);
   const AVCodec *codec = AvCodec_val(_codec);
-  CAMLreturn(value_of_video_codec_id(codec->id));
+  CAMLreturn(Val_VideoCodecID(codec->id));
 }
 
 CAMLprim value ocaml_avcodec_get_subtitle_codec_id(value _codec) {
   CAMLparam1(_codec);
   const AVCodec *codec = AvCodec_val(_codec);
-  CAMLreturn(value_of_subtitle_codec_id(codec->id));
+  CAMLreturn(Val_SubtitleCodecID(codec->id));
 }
 
 CAMLprim value ocaml_avcodec_get_audio_codec_id_name(value _codec_id) {
@@ -1097,28 +1074,28 @@ CAMLprim value ocaml_avcodec_find_audio_encoder_by_name(value _name) {
   CAMLparam1(_name);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_encoder_by_name(String_val(_name), AVMEDIA_TYPE_AUDIO)));
+      &ret, find_encoder_by_name(String_val(_name), AVMEDIA_TYPE_AUDIO)));
 }
 
 CAMLprim value ocaml_avcodec_find_audio_encoder(value _id) {
   CAMLparam1(_id);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_encoder(AudioCodecID_val(_id), AVMEDIA_TYPE_AUDIO)));
+      &ret, find_encoder(AudioCodecID_val(_id), AVMEDIA_TYPE_AUDIO)));
 }
 
 CAMLprim value ocaml_avcodec_find_audio_decoder_by_name(value _name) {
   CAMLparam1(_name);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_decoder_by_name(String_val(_name), AVMEDIA_TYPE_AUDIO)));
+      &ret, find_decoder_by_name(String_val(_name), AVMEDIA_TYPE_AUDIO)));
 }
 
 CAMLprim value ocaml_avcodec_find_audio_decoder(value _id) {
   CAMLparam1(_id);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_decoder(AudioCodecID_val(_id), AVMEDIA_TYPE_AUDIO)));
+      &ret, find_decoder(AudioCodecID_val(_id), AVMEDIA_TYPE_AUDIO)));
 }
 
 CAMLprim value ocaml_avcodec_name(value _codec) {
@@ -1337,7 +1314,7 @@ CAMLprim value ocaml_avcodec_get_supported_sample_rates(value _codec) {
 /**** Audio codec parameters ****/
 CAMLprim value ocaml_avcodec_parameters_get_audio_codec_id(value _cp) {
   CAMLparam1(_cp);
-  CAMLreturn(value_of_audio_codec_id(CodecParameters_val(_cp)->codec_id));
+  CAMLreturn(Val_AudioCodecID(CodecParameters_val(_cp)->codec_id));
 }
 
 CAMLprim value ocaml_avcodec_parameters_get_channel_layout(value _cp) {
@@ -1377,28 +1354,28 @@ CAMLprim value ocaml_avcodec_find_video_decoder_by_name(value _name) {
   CAMLparam1(_name);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_decoder_by_name(String_val(_name), AVMEDIA_TYPE_VIDEO)));
+      &ret, find_decoder_by_name(String_val(_name), AVMEDIA_TYPE_VIDEO)));
 }
 
 CAMLprim value ocaml_avcodec_find_video_decoder(value _id) {
   CAMLparam1(_id);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_decoder(VideoCodecID_val(_id), AVMEDIA_TYPE_VIDEO)));
+      &ret, find_decoder(VideoCodecID_val(_id), AVMEDIA_TYPE_VIDEO)));
 }
 
 CAMLprim value ocaml_avcodec_find_video_encoder_by_name(value _name) {
   CAMLparam1(_name);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_encoder_by_name(String_val(_name), AVMEDIA_TYPE_VIDEO)));
+      &ret, find_encoder_by_name(String_val(_name), AVMEDIA_TYPE_VIDEO)));
 }
 
 CAMLprim value ocaml_avcodec_find_video_encoder(value _id) {
   CAMLparam1(_id);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_encoder(VideoCodecID_val(_id), AVMEDIA_TYPE_VIDEO)));
+      &ret, find_encoder(VideoCodecID_val(_id), AVMEDIA_TYPE_VIDEO)));
 }
 
 CAMLprim value ocaml_avcodec_get_supported_frame_rates(value _codec) {
@@ -1436,7 +1413,7 @@ CAMLprim value ocaml_avcodec_get_supported_pixel_formats(value _codec) {
 /**** Video codec parameters ****/
 CAMLprim value ocaml_avcodec_parameters_get_video_codec_id(value _cp) {
   CAMLparam1(_cp);
-  CAMLreturn(value_of_video_codec_id(CodecParameters_val(_cp)->codec_id));
+  CAMLreturn(Val_VideoCodecID(CodecParameters_val(_cp)->codec_id));
 }
 
 CAMLprim value ocaml_avcodec_parameters_get_width(value _cp) {
@@ -1525,7 +1502,7 @@ CAMLprim value ocaml_avcodec_get_unknown_codec_id_name(value _codec_id) {
 
 CAMLprim value ocaml_avcodec_parameters_get_unknown_codec_id(value _cp) {
   CAMLparam1(_cp);
-  CAMLreturn(value_of_unknown_codec_id(CodecParameters_val(_cp)->codec_id));
+  CAMLreturn(Val_UnknownCodecID(CodecParameters_val(_cp)->codec_id));
 }
 
 /**** Subtitle codec ID ****/
@@ -1540,34 +1517,34 @@ CAMLprim value ocaml_avcodec_find_subtitle_decoder_by_name(value _name) {
   CAMLparam1(_name);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_decoder_by_name(String_val(_name), AVMEDIA_TYPE_SUBTITLE)));
+      &ret, find_decoder_by_name(String_val(_name), AVMEDIA_TYPE_SUBTITLE)));
 }
 
 CAMLprim value ocaml_avcodec_find_subtitle_decoder(value _id) {
   CAMLparam1(_id);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_decoder(SubtitleCodecID_val(_id), AVMEDIA_TYPE_SUBTITLE)));
+      &ret, find_decoder(SubtitleCodecID_val(_id), AVMEDIA_TYPE_SUBTITLE)));
 }
 
 CAMLprim value ocaml_avcodec_find_subtitle_encoder_by_name(value _name) {
   CAMLparam1(_name);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_encoder_by_name(String_val(_name), AVMEDIA_TYPE_SUBTITLE)));
+      &ret, find_encoder_by_name(String_val(_name), AVMEDIA_TYPE_SUBTITLE)));
 }
 
 CAMLprim value ocaml_avcodec_find_subtitle_encoder(value _id) {
   CAMLparam1(_id);
   CAMLlocal1(ret);
   CAMLreturn(value_of_avcodec(
-      ret, find_encoder(SubtitleCodecID_val(_id), AVMEDIA_TYPE_SUBTITLE)));
+      &ret, find_encoder(SubtitleCodecID_val(_id), AVMEDIA_TYPE_SUBTITLE)));
 }
 
 /**** Subtitle codec parameters ****/
 CAMLprim value ocaml_avcodec_parameters_get_subtitle_codec_id(value _cp) {
   CAMLparam1(_cp);
-  CAMLreturn(value_of_subtitle_codec_id(CodecParameters_val(_cp)->codec_id));
+  CAMLreturn(Val_SubtitleCodecID(CodecParameters_val(_cp)->codec_id));
 }
 
 CAMLprim value ocaml_avcodec_parameters_subtitle_copy(value _codec_id,
@@ -1646,10 +1623,10 @@ CAMLprim value ocaml_avcodec_get_next_codec(value h) {
   }
 
   _h = caml_alloc_tuple(1);
-  Store_field(_h, 0, value_of_avobj(_tmp, s));
+  Store_field(_h, 0, value_of_avobj(&_tmp, s));
 
   _ans = caml_alloc_tuple(4);
-  Store_field(_ans, 0, value_of_avcodec(_tmp, codec));
+  Store_field(_ans, 0, value_of_avcodec(&_tmp, codec));
   Store_field(_ans, 1, _id);
   Store_field(_ans, 2, Val_bool(av_codec_is_encoder(codec)));
   Store_field(_ans, 3, _h);
@@ -1710,7 +1687,7 @@ CAMLprim value ocaml_avcodec_bsf_next(value _cursor) {
     len++;
   }
   Store_field(ans, 1, tmp);
-  Store_field(ans, 2, value_of_avclass(tmp, filter->priv_class));
+  Store_field(ans, 2, value_of_avclass(&tmp, filter->priv_class));
 
   tmp = caml_alloc(1, Abstract_tag);
   BsfCursor_val(tmp) = cursor;
