@@ -1,7 +1,12 @@
 module C = Configurator.V1
 
+let os_type = ref ""
+
 let () =
-  C.main ~name:"ffmpeg-avdevice-pkg-config" (fun c ->
+  C.main
+    ~args:[("--os_type", String (fun s -> os_type := s), "")]
+    ~name:"ffmpeg-avdevice-pkg-config"
+    (fun c ->
       let default : C.Pkg_config.package_conf =
         { libs = ["-lavdevice"]; cflags = [] }
       in
@@ -16,5 +21,14 @@ let () =
                 | Error msg -> failwith msg
                 | Ok deps -> deps)
       in
+      let libs =
+        if !os_type = "Win32" then
+          List.filter
+            (fun flag ->
+              String.length flag < 3
+              || (String.sub flag 0 3 <> "-Wl" && flag <> "-static-libgcc"))
+            conf.libs
+        else conf.libs
+      in
       C.Flags.write_sexp "c_flags.sexp" conf.cflags;
-      C.Flags.write_sexp "c_library_flags.sexp" conf.libs)
+      C.Flags.write_sexp "c_library_flags.sexp" libs)
