@@ -15,9 +15,9 @@
 
 #include <libavutil/avassert.h>
 #include <libavutil/avstring.h>
+#include <libavutil/mem.h>
 #include <libavutil/pixdesc.h>
 #include <libavutil/pixfmt.h>
-#include <libavutil/mem.h>
 
 #include "avutil_stubs.h"
 #include "channel_layout_stubs.h"
@@ -261,7 +261,7 @@ CAMLprim value ocaml_ffmpeg_process_log(value cb) {
       caml_callback(cb, buffer);
 
       next_log_msg = log_msg->next;
-      free(log_msg);
+      av_free(log_msg);
       log_msg = next_log_msg;
     }
   }
@@ -286,7 +286,7 @@ static void av_log_ocaml_callback(void *ptr, int level, const char *fmt,
   }
 
   // TODO: check for NULL here
-  log_msg->next = malloc(sizeof(log_msg_t));
+  log_msg->next = av_malloc(sizeof(log_msg_t));
 
   log_msg = (log_msg_t *)log_msg->next;
   log_msg->next = NULL;
@@ -519,7 +519,7 @@ CAMLprim value ocaml_avutil_find_sample_fmt(value _name) {
 
   enum AVSampleFormat ret = av_get_sample_fmt(name);
 
-  free(name);
+  av_free(name);
 
   if (ret == AV_SAMPLE_FMT_NONE)
     caml_raise_not_found();
@@ -691,8 +691,7 @@ static struct custom_operations frame_ops = {
     "ocaml_avframe",     finalize_frame,           custom_compare_default,
     custom_hash_default, custom_serialize_default, custom_deserialize_default};
 
-value value_of_frame(AVFrame *frame) {
-  value ret;
+void value_of_frame(value *ret, AVFrame *frame) {
   if (!frame)
     Fail("Empty frame");
 
@@ -703,10 +702,8 @@ value value_of_frame(AVFrame *frame) {
     n++;
   }
 
-  ret = caml_alloc_custom_mem(&frame_ops, sizeof(AVFrame *), size);
-  Frame_val(ret) = frame;
-
-  return ret;
+  *ret = caml_alloc_custom_mem(&frame_ops, sizeof(AVFrame *), size);
+  Frame_val(*ret) = frame;
 }
 
 CAMLprim value ocaml_avutil_frame_pts(value _frame) {
@@ -870,6 +867,7 @@ CAMLprim value ocaml_avutil_frame_copy(value _src, value _dst) {
 CAMLprim value ocaml_avutil_video_create_frame(value _w, value _h,
                                                value _format) {
   CAMLparam1(_format);
+  CAMLlocal1(ans);
   AVFrame *frame = av_frame_alloc();
   if (!frame)
     caml_raise_out_of_memory();
@@ -885,7 +883,9 @@ CAMLprim value ocaml_avutil_video_create_frame(value _w, value _h,
     ocaml_avutil_raise_error(ret);
   }
 
-  CAMLreturn(value_of_frame(frame));
+  value_of_frame(&ans, frame);
+
+  CAMLreturn(ans);
 }
 
 /* Adapted from alloc_audio_frame */
@@ -894,6 +894,7 @@ CAMLprim value ocaml_avutil_audio_create_frame(value _sample_fmt,
                                                value _samplerate,
                                                value _samples) {
   CAMLparam2(_sample_fmt, _channel_layout);
+  CAMLlocal1(ans);
   enum AVSampleFormat sample_fmt = SampleFormat_val(_sample_fmt);
   AVChannelLayout *channel_layout = AVChannelLayout_val(_channel_layout);
   int sample_rate = Int_val(_samplerate);
@@ -923,7 +924,9 @@ CAMLprim value ocaml_avutil_audio_create_frame(value _sample_fmt,
     ocaml_avutil_raise_error(ret);
   }
 
-  CAMLreturn(value_of_frame(frame));
+  value_of_frame(&ans, frame);
+
+  CAMLreturn(ans);
 }
 
 CAMLprim value ocaml_avutil_audio_frame_get_sample_format(value _frame) {
@@ -1047,9 +1050,9 @@ CAMLprim value ocaml_avutil_video_frame_get_linesize(value _frame,
   int line = Int_val(_line);
 
   if (line < 0 || line >= AV_NUM_DATA_POINTERS || !frame->data[line])
-    Fail(
-        "Failed to get linesize from video frame : line (%d) out of boundaries",
-        line);
+    Fail("Failed to get linesize from video frame : line (%d) out of "
+         "boundaries",
+         line);
 
   CAMLreturn(Val_int(frame->linesize[line]));
 }
@@ -1094,22 +1097,19 @@ CAMLprim value ocaml_avutil_video_get_frame_bigarray_planes(
 void static finalize_subtitle(value v) {
   struct AVSubtitle *subtitle = Subtitle_val(v);
   avsubtitle_free(subtitle);
-  free(subtitle);
+  av_free(subtitle);
 }
 
 static struct custom_operations subtitle_ops = {
     "ocaml_avsubtitle",  finalize_subtitle,        custom_compare_default,
     custom_hash_default, custom_serialize_default, custom_deserialize_default};
 
-value value_of_subtitle(AVSubtitle *subtitle) {
-  value ret;
+void value_of_subtitle(value *ret, AVSubtitle *subtitle) {
   if (!subtitle)
     Fail("Empty subtitle");
 
-  ret = caml_alloc_custom(&subtitle_ops, sizeof(AVSubtitle *), 0, 1);
-  Subtitle_val(ret) = subtitle;
-
-  return ret;
+  *ret = caml_alloc_custom(&subtitle_ops, sizeof(AVSubtitle *), 0, 1);
+  Subtitle_val(*ret) = subtitle;
 }
 
 int subtitle_header_default(AVCodecContext *codec_context) { return 0; }
@@ -1123,11 +1123,11 @@ CAMLprim value ocaml_avutil_subtitle_create_frame(value _start_time,
   int64_t end_time = Int64_val(_end_time);
   int nb_lines = Wosize_val(_lines);
 
-  AVSubtitle *subtitle = (AVSubtitle *)calloc(1, sizeof(AVSubtitle));
+  AVSubtitle *subtitle = (AVSubtitle *)av_mallocz(sizeof(AVSubtitle));
   if (!subtitle)
     caml_raise_out_of_memory();
 
-  ans = value_of_subtitle(subtitle);
+  value_of_subtitle(&ans, subtitle);
 
   //  subtitle->start_display_time = (uint32_t)start_time;
   subtitle->end_display_time = (uint32_t)end_time;
@@ -1407,7 +1407,7 @@ CAMLprim value ocaml_avutil_av_opt_iter(value _cursor, value _class) {
   switch (option->type) {
   case AV_OPT_TYPE_CONST:
     _type = PVV_Constant;
-    Store_field(_tmp, 0, value_of_avoptions(_cursor, option));
+    Store_field(_tmp, 0, value_of_avoptions(&_cursor, option));
     Store_field(_spec, 0, _tmp);
     break;
   case AV_OPT_TYPE_BOOL:
@@ -1582,10 +1582,11 @@ CAMLprim value ocaml_avutil_av_opt_iter(value _cursor, value _class) {
   Store_field(Field(_tmp, 0), 0, value_of_avoptions(_cursor, option));
 #else
   Store_field(Field(_tmp, 0), 0, caml_alloc_tuple(2));
-  Store_field(Field(Field(_tmp, 0), 0), 0, value_of_avobj(_cursor, cursor));
-  Store_field(Field(Field(_tmp, 0), 0), 1, value_of_avoptions(_cursor, option));
+  Store_field(Field(Field(_tmp, 0), 0), 0, value_of_avobj(&_cursor, cursor));
+  Store_field(Field(Field(_tmp, 0), 0), 1,
+              value_of_avoptions(&_cursor, option));
 #endif
-  Store_field(Field(_tmp, 0), 1, value_of_avclass(_class, class));
+  Store_field(Field(_tmp, 0), 1, value_of_avclass(&_class, class));
   Store_field(_opt, 5, _tmp);
 
   if (unimplement_option)
