@@ -16,20 +16,21 @@ let () =
     try
       let video_idx, video_src, _ = Av.find_best_video_stream src in
       let video_dst = Avdevice.open_video_output "sdl2" in
-      (Some (video_idx, video_dst), [video_src])
+      let _, video_stream, _ = List.hd (Av.get_video_streams video_dst) in
+      (Some (video_idx, video_dst, video_stream), [video_src])
     with Avutil.Error _ -> (None, [])
   in
 
   let rec f () =
     match (Av.read_input ~video_frame src, video) with
-      | `Video_frame (i, frame), Some (idx, dst) when i = idx ->
-          Av.write_video_frame dst frame;
+      | `Video_frame (i, frame), Some (idx, _, stream) when i = idx ->
+          Av.write_frame stream frame;
           f ()
       | exception Avutil.Error `Eof -> ()
       | _ -> f ()
   in
   f ();
   Av.close src;
-  let () = match video with Some (_, dst) -> Av.close dst | None -> () in
+  let () = match video with Some (_, dst, _) -> Av.close dst | None -> () in
   Gc.full_major ();
   Gc.full_major ()
